# Dependabot configuration for automatic dependency updates
# https://docs.github.com/en/code-security/dependabot/dependabot-version-updates/configuration-options-for-the-dependabot.yml-file

version: 2
updates:
  # Python dependencies (pip/poetry)
  - package-ecosystem: "pip"
    directory: "/"
    schedule:
      interval: "{{ dependabot.schedule }}"
    open-pull-requests-limit: {{ dependabot.open_pull_requests_limit }}
    commit-message:
      prefix: "deps"
    labels:
      - "dependencies"
      - "python"

  # GitHub Actions
  - package-ecosystem: "github-actions"
    directory: "/"
    schedule:
      interval: "{{ dependabot.schedule }}"
    open-pull-requests-limit: {{ dependabot.open_pull_requests_limit }}
    commit-message:
      prefix: "ci"
    labels:
      - "dependencies"
      - "github-actions"
