# Python .gitignore
# Generated by pypreset augment

# Byte-compiled / optimized / DLL files
__pycache__/
*.py[cod]
*$py.class

# C extensions
*.so

# Distribution / packaging
.Python
build/
develop-eggs/
dist/
downloads/
eggs/
.eggs/
lib/
lib64/
parts/
sdist/
var/
wheels/
*.egg-info/
.installed.cfg
*.egg

# PyInstaller
*.manifest
*.spec

# Installer logs
pip-log.txt
pip-delete-this-directory.txt

# Unit test / coverage reports
htmlcov/
.tox/
.nox/
.coverage
.coverage.*
.cache
nosetests.xml
coverage.xml
*.cover
*.py,cover
.hypothesis/
.pytest_cache/

# Translations
*.mo
*.pot

# Environments
.env
.venv
env/
venv/
ENV/
env.bak/
venv.bak/
.python-version

# IDE / Editor
.idea/
.vscode/
*.swp
*.swo
*~
.project
.pydevproject
.settings/
*.sublime-project
*.sublime-workspace

# Jupyter Notebook
.ipynb_checkpoints

# pytype static type analyzer
.pytype/

# mypy
.mypy_cache/
.dmypy.json
dmypy.json

# ruff
.ruff_cache/

# pyright
pyrightconfig.json

# Pyre type checker
.pyre/

{% if package_manager == "poetry" %}
# Poetry
# Uncomment the following line if you want to track the lock file
# poetry.lock
{% elif package_manager == "pdm" %}
# PDM
# Uncomment the following line if you want to track the lock file
# pdm.lock
__pypackages__/
{% endif %}

# OS-specific files
.DS_Store
Thumbs.db

# Local development
*.local
.envrc

# Logs
*.log
logs/
