"""Basic tests for {{ project.name }}.

Generated by pypreset augment.
"""

{% if has_src_layout %}
import {{ project.package_name }}
{% endif %}


class Test{{ project.package_name | title | replace('_', '') }}:
    """Test suite for {{ project.name }}."""

{% if has_src_layout %}
    def test_version_exists(self) -> None:
        """Test that version is defined."""
        assert hasattr({{ project.package_name }}, "__version__")
        assert isinstance({{ project.package_name }}.__version__, str)

    def test_import(self) -> None:
        """Test that package can be imported."""
        assert {{ project.package_name }} is not None
{% else %}
    def test_placeholder(self) -> None:
        """Placeholder test - replace with actual tests."""
        assert True
{% endif %}


def test_example() -> None:
    """Example test function.

    Replace this with actual tests for your package.
    """
    # Arrange
    expected = 4

    # Act
    result = 2 + 2

    # Assert
    assert result == expected
